/* Version 0.2B */
/* /hs request, should reduce need for opers to set vhosts */
/* Aliased to !vhost */
/* Uses host tracking */

#include "module.h"
#define AUTHOR "Alek"
#define VERSION "0.2B"

#define VHOST_TIME "10080" /* Needs to be 10080 */

/* Host structure to keep track of who can and cannot set vhosts */

typedef struct hostinfo_ HostInfo;

struct hostinfo_ {
	HostInfo *next, *prev;
	char *nick;
	char *host;
	int32 time;
};

#define HASH(nick)	((tolower((nick)[0])&31)<<5 | (tolower((nick)[1])&31))

HostInfo *hilists[1024];

HostInfo *findinfo(const char *host){
    HostInfo *hi;
    
    for (hi = hilists[HASH(host)]; hi; hi = hi->next) {
        if (stricmp(hi->host, host) == 0)
            return hi;
    }

    return NULL;
}

/* Insert into struct */
HostInfo *insert_info(char *host){
        HostInfo *hi;
        int index;
        hi = scalloc(1, sizeof(HostInfo));
        hi->host = sstrdup(host);
        index = HASH(hi->host);

        hi->prev = NULL;
        hi->next = hilists[index];
    if (hi->next)
        hi->next->prev = hi;
        hilists[index] = hi;
        
        return hi;
}

/* Request command */
int hs_request(User *u);
int unban(int ac, char **argv);
/* Privmsg hook */
int my_privmsg(char *source, int ac, char **msg);

int AnopeInit(void) {
	Command *c;
	Message *msg;
	
    c = createCommand("request", hs_request, NULL, -1, -1, -1, -1, -1);
  	alog("Adding request command: %d", moduleAddCommand(HOSTSERV, c, MOD_HEAD));
	
	msg = createMessage("PRIVMSG", my_privmsg);
	alog("Hooking into privmsg: %d", moduleAddMessage(msg, MOD_HEAD));
	moduleAddAuthor(AUTHOR);
	moduleAddVersion(VERSION);
	send_cmd(s_HostServ, "join #vhost");
	return MOD_CONT;
}

void AnopeFini(void){
 alog("hs_request.so: unloading");   
}

int hs_request(User *u){
    char *rawhostmask = strtok(NULL, " ");
    char *hostmask = smalloc(HOSTMAX);

    NickAlias *na;
    int32 tmp_time;
    char *s;
    HostInfo *hi;
    char *vIdent = NULL;

    if (!(na = findnick(u->nick)))
        return MOD_CONT;
        
    if (!(hi = findinfo(u->host)))
        hi = insert_info(u->host);
    else if (hi->time > (time(NULL) - dotime(VHOST_TIME))){
          notice(s_HostServ, u->nick, "Error: You must wait 1 week from your last request");
          return MOD_CONT;
    }

    if (!rawhostmask) {
        notice(s_HostServ, u->nick, "Syntax: /msg hostserv REQUEST <vhost>");
        return MOD_CONT;
    }

    vIdent = myStrGetOnlyToken(rawhostmask, '@', 0);    /* Get the first substring, @ as delimiter */
    if (vIdent) {
        rawhostmask = myStrGetTokenRemainder(rawhostmask, '@', 1);      /* get the remaining string */
        if (!rawhostmask) {
            notice_lang(s_HostServ, u, HOST_SETALL_SYNTAX, s_HostServ);
            return MOD_CONT;
        }
        if (strlen(vIdent) > USERMAX - 1) {
            notice_lang(s_HostServ, u, HOST_SET_IDENTTOOLONG, USERMAX);
            return MOD_CONT;
        } else {
            for (s = vIdent; *s; s++) {
                if (!isvalidchar(*s)) {
                    notice_lang(s_HostServ, u, HOST_SET_IDENT_ERROR);
                    return MOD_CONT;
                }
            }
        }
#ifndef HAS_VIDENT
        notice_lang(s_HostServ, u, HOST_NO_VIDENT);
        return MOD_CONT;
#endif
    }

    if (strlen(rawhostmask) < HOSTMAX - 1)
        snprintf(hostmask, HOSTMAX - 1, "%s", rawhostmask);
    else {
        notice_lang(s_HostServ, u, HOST_SET_TOOLONG, HOSTMAX);
        return MOD_CONT;
    }

    if (!isValidHost(hostmask, 3)) {
        notice_lang(s_HostServ, u, HOST_SET_ERROR);
        return MOD_CONT;
    }

    tmp_time = time(NULL);

    if (match_wild_nocase("*rizon*", hostmask) || 
        match_wild_nocase("*oper*", hostmask) ||
        match_wild_nocase("*admin*",hostmask) ||
        match_wild_nocase("*.gov", hostmask) || 
        match_wild_nocase("*service*", hostmask) || 
        match_wild_nocase("*staff*", hostmask) ||
        match_wild_nocase("*packet*", hostmask) ||
        match_wild_nocase("*ddos*",hostmask) ||
        match_wild_nocase("*dd0s*", hostmask) ||
        match_wild_nocase("*nsa*", hostmask) ||
        match_wild_nocase("*mpaa*", hostmask) ||
        match_wild_nocase("*ddos*",hostmask) ||
        match_wild_nocase("*riaa*", hostmask) ||
        match_wild_nocase("*kkk*", hostmask) ||
        match_wild_nocase("*cia*", hostmask) ||
        match_wild_nocase("*botnet", hostmask)){
           notice(s_HostServ, u->nick, "Error: That host contains a restricted word. Please try another.");
           
           return MOD_CONT;
        }


    if ((na = findnick(u->nick))) {
        notice(s_HostServ, u->nick, "You vhost has been set, /hs on to activate"); 
        do_hs_sync(na->nc, vIdent, hostmask, u->nick, tmp_time);
    }
    hi->time = time(NULL);
    free(hostmask);
    return MOD_CONT;
}

int my_privmsg(char *source, int ac, char **msg){
    User *u;
    HostInfo *hi;
    char *cmd, mask[BUFSIZE], *argv[0];
    ChannelInfo *ci;

    if (ac != 2)
        return MOD_CONT;
    
    if (!(u = finduser(source)))
        return MOD_CONT;
    
    if (stricmp("#vhost", msg[0]))
        return MOD_CONT;

    if (!(ci = cs_findchan(msg[0])))
	return MOD_CONT; 

    cmd = myStrGetToken(msg[1], ' ', 0);
    if (!cmd)
        return MOD_CONT;
         
    if (!stricmp("!vhost", cmd)){
        cmd = strtok(msg[1], " ");
        hs_request(u);
        if ((hi = findinfo(u->host)))
          if ((hi->time > (time(NULL) - dotime(VHOST_TIME)))){
            get_idealban(ci, u, mask, sizeof(mask));
	    send_cmd(s_HostServ, "MODE #vhost +b %s", mask);
	    send_cmd(s_HostServ, "KICK #vhost %s :You can request a new vhost a week after your last one. The new request system is based on hosts, if you have not recently requested a vhost, please rejoin #vhost and request assistance from an IRCop", u->nick);  
        argv[0] = sstrdup(mask);
        moduleAddCallback("unban", time(NULL) + 15, unban, 1, argv);
	  }    
    }
	return MOD_CONT;
}

int unban(int ac, char **argv){
    send_cmd(s_HostServ, "MODE #vhost -b %s", argv[0]);
    return 0;
}
