#include "module.h"
#define AUTHOR "SGR" 
#define VERSION "1.05"

  /* ----------------------------------------------------------- 
   * Name: ns_checkemailonregister 
   * Authors: SGR     <Alex_SGR@ntlworld.com>, 
   *          Certus  <certus@anope.org>,
   *          Nima    <joris@anope.org>
   * Date: 17/11/2003 
   * ----------------------------------------------------------- 
   * Functions: m_ns_notice_noreg_due_to_mail, m_ns_check_email
   * Limitations: NONE KNOWN. 
   * Tested: Ultimate(2.8.x), Unreal(3.2) 
   * ----------------------------------------------------------- 
   * This version has been tested on Ultimate2.8.6, viagra 
   * Unreal and Bahamut. All IRCd's should be compatible with 
   * this module. 
   *   
   * This module will prevent users registering nicks using an
   * e-mail address who's host is either in the one of the 3 
   * "AlwaysBlock" feilds, or appears in the restrictemail.dat
   * file that should be placed in the same dir as the services
   * databases. The 'host file' is optional.
   * 
   *  Please set the config options below. 
   * 
   *  To DISABLE the notices comment out the #DEFINE lines. 
   *  #define DENY_NICK_REG_AS_DODGY_EMAIL1-6 "Notice text". 
   *  i.e 
   *  #define DENY_NICK_REG_AS_DODGY_EMAIL1 "notice text" becomes 
   */  
        // #define DENY_NICK_REG_AS_DODGY_EMAIL1 "notice text"  
   /* -----------------------------------------------------------
    * ChangeLog:
    *  
    *   1: Core
    *   2: Added 'restrictemail.dat' and ALWAYSBLOCK options.
    *   3: Added oper-exempt from registration restriction code.
    *   4: Fixed minor file-closing bug. [1.01]
    *   5: Removed an error message from a place where there was 
    *      no error.
    *   
    *   
    * -----------------------------------------------------------*/
    
/* ---------------------------------------------------------------------- */ 
/* START OF CONFIGURATION BLOCK - please read the comments :)             */ 
/* ---------------------------------------------------------------------- */ 

/* These are the messages noticed to people who attempt to register using 
 * a denied e-mail address. */

#define DENY_NICK_REG_AS_DODGY_EMAIL1 "The e-mail address you submitted is from a provider"
#define DENY_NICK_REG_AS_DODGY_EMAIL2 "that is in our blacklist. Due to abuse, we do not"
#define DENY_NICK_REG_AS_DODGY_EMAIL3 "accept addresses located in this list."
#define DENY_NICK_REG_AS_DODGY_EMAIL4 "Services will not respond to any more requests for 1 minute."
#define DENY_NICK_REG_AS_DODGY_EMAIL5 "Please try again then with a different e-mail address." 
//  #define DENY_NICK_REG_AS_DODGY_EMAIL6 "w00tage"

/* These domains will _ALWAYS_ be blocked from non-opers when the module 
 * is loaded, regardless of wether they are in the restrictemail.dat file
 * or not */

//#define ALWAYSBLOCK1 "hotmail.com"
//#define ALWAYSBLOCK2 "mail.com"
//#define ALWAYSBLOCK3 "gmail.com"

/* This option will set NickServ to KILL (1) or AKILL (2) clients that 
 * attempt to register with a phrobited e-mail address. PLEASE DO NOT
 * SET THIS UNLESS ABSOLUTLY ESSENTIAL!!! 
 *
 *  0 = No action except any notices defined above
 *  1 = Add Services IGNORE for user for 15mins
 *  2 = KILL  users using OWN_USERS_USING_DODGY_EMAILS_REASON 
 *  3 = AKILL users using OWN_USERS_USING_DODGY_EMAILS_REASON for
 *      OWN_USERS_USING_DODGY_EMAILS_REASON_TIME
 */

#define OWN_USERS_USING_DODGY_EMAILS              0 
#define OWN_USERS_USING_DODGY_EMAILS_REASON       "Attempted to register using phrobited E-Mail address. Possible trojan spammer. E-mail kline@rizon.net if you belive this is a mistake."
#define OWN_USERS_USING_DODGY_EMAILS_REASON_TIME  5

/* ---------------------------------------------------------------------- */
/* END OF CONFIGURATION BLOCK - Don't edit beyond this point unless you   */
/* KNOW what you are doing.                                               */
/* ---------------------------------------------------------------------- */ 


int m_ns_check_email(User *);
void m_ns_notice_noreg_due_to_mail(User *u);
int CheckRestrictEmail(char *to_be_matched);
int m_ns_emailtakeaction;
int m_ns_emailtakeactiontime;

int AnopeInit(int argc, char **argv)
{ 
        Command *c; 
        alog("Loading module ns_checkemailonregister.so"); 
        c = createCommand("REGISTER", m_ns_check_email, NULL,-1,-1,-1,-1,-1); 
        moduleAddCommand(NICKSERV, c, MOD_HEAD); 
        alog("[ns_checkemailonregister] %s registrations using phrobited e-mail addresses will", s_NickServ); 
        alog("[ns_checkemailonregister] now be rejected from non-opers."); 
        moduleAddAuthor(AUTHOR);
        moduleAddVersion(VERSION);
	m_ns_emailtakeaction     = OWN_USERS_USING_DODGY_EMAILS;
	m_ns_emailtakeactiontime = OWN_USERS_USING_DODGY_EMAILS_REASON_TIME;
	alog("[ns_checkemailonregister] Yayness! (TM) Module loaded and active!"); 
        return MOD_CONT; 
} 

int m_ns_check_email(User *u)
{
   char *tmp;
   char *pass;
   char *email;
   char *host;
   char dilim = ' ';
   tmp = moduleGetLastBuffer();
   if (is_oper(u)) {
       return MOD_CONT;
   }
   if (tmp == NULL) {
       return MOD_STOP;
   }
   pass = myStrGetOnlyToken(tmp, dilim, 0);
   email = myStrGetTokenRemainder(tmp, dilim,1);
   if (pass == NULL || email == NULL) {
       return MOD_CONT;
   }
   host = (char *)strchr(email, '@');
   if (host == NULL) {
       return MOD_CONT;
   }
   *host++;
   #ifdef ALWAYSBLOCK1
   if (!stricmp(host,ALWAYSBLOCK1)) {
       m_ns_notice_noreg_due_to_mail(u);
       return MOD_STOP; 
   }
   #endif
   #ifdef ALWAYSBLOCK2
   if (!stricmp(host,ALWAYSBLOCK2)) {
       m_ns_notice_noreg_due_to_mail(u);
       return MOD_STOP; 
   }
   #endif
   #ifdef ALWAYSBLOCK3
   if (!stricmp(host,ALWAYSBLOCK3)) {
       m_ns_notice_noreg_due_to_mail(u);
       return MOD_STOP; 
   }
   #endif
   if (CheckRestrictEmail(host)) {
       m_ns_notice_noreg_due_to_mail(u);
       return MOD_STOP; 
   } 
   else {
          return MOD_CONT; 
   }
   return MOD_CONT;
}

void AnopeFini(void) {
        alog("Unloading ns_check_email.so");
}

int CheckRestrictEmail(char *to_be_matched)
{
   FILE *RestrictEmailFile;
   char definition_string[128];
   int matched = 0;
   int len = 0;
   /* Open the file (r == in read only mode) */
   RestrictEmailFile = fopen("restrictemail.dat","r");
   /* if the file is there, loop until there is no
    * more data to be read from the file */
   if (RestrictEmailFile) {
       while (fgets(definition_string, 127, RestrictEmailFile) != NULL) {
              len = strlen(definition_string);
              /* replace the \n character with \0 */
              definition_string[len-1]='\0';
	      //alog("%d %s", len, definition_string);
              if (!stricmp(definition_string,to_be_matched)) {
                  if (debug) {
                      alog("E-mail Restriction MATCH: %s +[MATCHED WITH]+ %s", to_be_matched, definition_string);
                  }
		  matched = 1;
		  fclose(RestrictEmailFile);
                  return 1;
                  break;
              }
       }
       if (matched) {
           fclose(RestrictEmailFile);
           return 1;
       }
   } 
   else {
         if (debug) {
                alog("[ns_checkemailonregister] CRITICAL ERROR: Unable to open restrictemail.dat [or there isn't one]");
         }
         return 0;
   }
   fclose(RestrictEmailFile);
   return 0;
}

void m_ns_notice_noreg_due_to_mail(User *u) 
{
    char mask[USERMAX + HOSTMAX + 2];
    
    if (m_ns_emailtakeaction) {
        if (m_ns_emailtakeaction == 1) {
	    if (!m_ns_emailtakeactiontime) {
	        m_ns_emailtakeactiontime = 900;
            }
            add_ignore(u->nick,time(NULL)+m_ns_emailtakeactiontime);
	    return;
        }
        if (m_ns_emailtakeaction == 2) {
            kill_user(s_NickServ, u->nick, OWN_USERS_USING_DODGY_EMAILS_REASON);
	    return;
        }
	if (m_ns_emailtakeaction == 3) {
            sprintf(mask, "*@%s", u->host);
	    if (!m_ns_emailtakeactiontime) {
	        m_ns_emailtakeactiontime = 900;
            }
	    add_akill(NULL, mask, s_NickServ, time(NULL)+m_ns_emailtakeactiontime, OWN_USERS_USING_DODGY_EMAILS_REASON);
            if (WallOSAkill) {
                alog(s_NickServ, "%s used AKILL on %s (%s)", s_NickServ, u->nick, OWN_USERS_USING_DODGY_EMAILS_REASON);
            }
	    if (!AkillOnAdd) {
                kill_user(s_NickServ, u->nick, OWN_USERS_USING_DODGY_EMAILS_REASON);
	    }
	    return;
        }
    }
    #ifdef DENY_NICK_REG_AS_DODGY_EMAIL1 
    notice(s_NickServ, u->nick, DENY_NICK_REG_AS_DODGY_EMAIL1); 
    #endif 
    #ifdef DENY_NICK_REG_AS_DODGY_EMAIL2 
    notice(s_NickServ, u->nick, DENY_NICK_REG_AS_DODGY_EMAIL2); 
    #endif 
    #ifdef DENY_NICK_REG_AS_DODGY_EMAIL3 
    notice(s_NickServ, u->nick, DENY_NICK_REG_AS_DODGY_EMAIL3); 
    #endif 
    #ifdef DENY_NICK_REG_AS_DODGY_EMAIL4 
    notice(s_NickServ, u->nick, DENY_NICK_REG_AS_DODGY_EMAIL4); 
    #endif 
    #ifdef DENY_NICK_REG_AS_DODGY_EMAIL5 
    notice(s_NickServ, u->nick, DENY_NICK_REG_AS_DODGY_EMAIL5); 
    #endif 
    #ifdef DENY_NICK_REG_AS_DODGY_EMAIL6 
    notice(s_NickServ, u->nick, DENY_NICK_REG_AS_DODGY_EMAIL6); 
    #endif 
    return;
}
